@@ -1,6 +1,6 @@
 from abc import ABCMeta, abstractmethod
 from collections.abc import AsyncGenerator, Awaitable, Coroutine
-from typing import Any, Callable, Generic, Optional, TypeVar, Union, overload, runtime_checkable
+from typing import Any, Callable, Generic, Optional, Union, overload, runtime_checkable
 
 from PIL import Image
 from reactivex import Observable
@@ -9,6 +9,7 @@ from typing_extensions import (
     NotRequired,
     Protocol,
     TypedDict,
+    TypeVar,
 )
 
 Callback = Union[
@@ -18,7 +19,7 @@ Callback = Union[
 
 
 # schema
-JsonSchemaType = Literal["string", "number", "boolean", "object", "array"]
+JsonSchemaType = Literal["string", "number", "boolean", "object", "array", "button"]
 
 
 class JsonBaseSchema(TypedDict):
@@ -32,59 +33,12 @@ class JsonBaseSchema(TypedDict):
     group: NotRequired[Optional[str]]
     defaultValue: NotRequired[Optional[Union[str, int, bool]]]
     store: NotRequired[Optional[bool]]
-
-
-class PluginJsonBaseSchema(JsonBaseSchema):
     onSet: NotRequired[
         Union[Callable[[Any, Any], Awaitable[Union[None, Any]]], Callable[[Any, Any], Union[None, Any]]]
-    ]
-    onGet: NotRequired[Union[Callable[[], Awaitable[Union[Any, None]]], Callable[[], Union[Any, None]]]]
-
-
-class PluginJsonSchemaString(PluginJsonBaseSchema):
-    type: Literal["string"]
-    format: NotRequired[str]
-    minLength: NotRequired[int]
-    maxLength: NotRequired[int]
-
-
-class PluginJsonSchemaNumber(PluginJsonBaseSchema):
-    type: Literal["number"]
-    minimum: NotRequired[int]
-    maximum: NotRequired[int]
-    step: NotRequired[float]
-
-
-class PluginJsonSchemaBoolean(PluginJsonBaseSchema):
-    type: Literal["boolean"]
-
-
-class PluginJsonSchemaEnum(PluginJsonBaseSchema):
-    type: Literal["string"]
-    enum: list[str]
-    multiple: NotRequired[bool]
-
-
-class PluginJsonSchemaObject(PluginJsonBaseSchema):
-    type: Literal["object"]
-    opened: NotRequired[bool]
-    properties: NotRequired["PluginJsonSchemaForm"]
-
-
-PluginJsonSchema = Union[
-    PluginJsonSchemaString,
-    PluginJsonSchemaNumber,
-    PluginJsonSchemaBoolean,
-    PluginJsonSchemaEnum,
-    PluginJsonSchemaObject,
-]
-
-
-PluginJsonSchemaForm = dict[str, PluginJsonSchema]
-
-
-class PluginRootSchema(TypedDict):
-    schema: PluginJsonSchemaForm
+    ]  # only for plugins
+    onGet: NotRequired[
+        Union[Callable[[], Awaitable[Union[Any, None]]], Callable[[], Union[Any, None]]]
+    ]  # only for plugins
 
 
 class JsonSchemaString(JsonBaseSchema):
@@ -113,25 +67,29 @@ class JsonSchemaEnum(JsonBaseSchema):
     multiple: NotRequired[bool]
 
 
+class JsonSchemaButton(JsonBaseSchema):
+    type: Literal["button"]
+
+
 class JsonSchemaObject(JsonBaseSchema):
     type: Literal["object"]
     opened: NotRequired[bool]
     properties: NotRequired["JsonSchemaForm"]
 
 
-class JsonSchemaButton(TypedDict):
+class JsonSchemaArray(JsonBaseSchema):
+    type: Literal["array"]
+    opened: NotRequired[bool]
+    items: NotRequired["JsonSchema"]
+
+
+class JsonSchemaObjectButton(TypedDict):
     label: str
     onSubmit: str
 
 
 class JsonSchemaObjectWithButtons(JsonSchemaObject):
-    buttons: tuple[JsonSchemaButton, NotRequired[JsonSchemaButton]]
-
-
-class JsonSchemaArray(JsonBaseSchema):
-    type: Literal["array"]
-    opened: NotRequired[bool]
-    items: NotRequired["JsonSchema"]
+    buttons: list[JsonSchemaObjectButton]
 
 
 JsonSchema = Union[
@@ -142,6 +100,7 @@ JsonSchema = Union[
     JsonSchemaObject,
     JsonSchemaObjectWithButtons,
     JsonSchemaArray,
+    JsonSchemaButton,
 ]
 
 
@@ -153,8 +112,8 @@ class RootSchema(TypedDict):
 
 
 class SchemaConfig(TypedDict):
-    rootSchema: PluginRootSchema
-    config: dict[str, Union[str, int, bool]]
+    rootSchema: RootSchema
+    config: dict[str, Any]
 
 
 # camera object
@@ -240,12 +199,12 @@ class DetectionZone(TypedDict):
 
 
 class CameraInformation(TypedDict):
-    model: str
-    manufacturer: str
-    hardware: str
-    serialNumber: str
-    firmwareVersion: str
-    supportUrl: str
+    model: NotRequired[str]
+    manufacturer: NotRequired[str]
+    hardware: NotRequired[str]
+    serialNumber: NotRequired[str]
+    firmwareVersion: NotRequired[str]
+    supportUrl: NotRequired[str]
 
 
 class MotionDetectionSettings(TypedDict):
@@ -1037,47 +996,52 @@ class DeviceManager(Protocol):
 
 # camera storage
 P = TypeVar("P")
+CS = TypeVar("CS", default=dict[str, Any])
 
 
 @runtime_checkable
-class CameraStorage(Protocol):
-    values: dict[str, Any]
+class CameraStorage(Protocol, Generic[CS]):
+    values: CS
+    schema: JsonSchemaForm
 
-    async def initialize_storage(self) -> None: ...
+    async def initializeStorage(self) -> None: ...
     @overload
-    def get_value(self, path: str) -> Union[Awaitable[P], P, None]: ...
+    def getValue(self, path: str) -> Union[Awaitable[P], P, None]: ...
     @overload
-    def get_value(self, path: str, default_value: P) -> Union[Awaitable[P], P]: ...
-    def get_value(self, path: str, default_value: Optional[P] = None) -> Union[Awaitable[P], P, None]: ...
-    async def set_value(self, path: str, new_value: Any) -> None: ...
-    def has_value(self, path: str) -> bool: ...
+    def getValue(self, path: str, default_value: P) -> Union[Awaitable[P], P]: ...
+    def getValue(self, path: str, default_value: Optional[P] = None) -> Union[Awaitable[P], P, None]: ...
+    async def setValue(self, path: str, new_value: Any) -> None: ...
+    def hasValue(self, path: str) -> bool: ...
     async def getConfig(self) -> SchemaConfig: ...
-    async def setConfig(self, new_config: dict[str, Any]) -> None: ...
+    async def setConfig(self, new_config: CS) -> None: ...
     @overload
-    async def add_schema(self, schema_or_path: PluginJsonSchemaForm) -> None: ...
+    async def addSchema(self, schema_or_path: JsonSchemaForm) -> None: ...
     @overload
-    async def add_schema(self, schema_or_path: str, schema: PluginJsonSchema) -> None: ...
-    async def add_schema(
+    async def addSchema(self, schema_or_path: str, schema: JsonSchema) -> None: ...
+    async def addSchema(
         self,
-        schema_or_path: Union[PluginJsonSchemaForm, str],
-        schema: Optional[PluginJsonSchema] = None,
+        schema_or_path: Union[JsonSchemaForm, str],
+        schema: Optional[JsonSchema] = None,
     ) -> None: ...
-    def remove_schema(self, path: str) -> None: ...
-    async def change_schema(self, path: str, new_schema: dict[str, Any]) -> None: ...
-    def get_schema(self, path: str) -> Optional[PluginJsonSchema]: ...
-    def has_schema(self, path: str) -> bool: ...
+    def removeSchema(self, path: str) -> None: ...
+    async def changeSchema(self, path: str, new_schema: dict[str, Any]) -> None: ...
+    def getSchema(self, path: str) -> Optional[JsonSchema]: ...
+    def hasSchema(self, path: str) -> bool: ...
+
+
+SC = TypeVar("SC", default=CameraStorage[dict[str, Any]], covariant=True)
 
 
 # StorageController
 @runtime_checkable
-class StorageController(Protocol):
+class StorageController(Protocol[SC]):
     def create_camera_storage(
         self,
         instance: Any,
         camera_id: str,
-        schema: Optional[PluginJsonSchemaForm] = None,
-    ) -> CameraStorage: ...
-    def get_camera_storage(self, camera_id: str) -> Optional[CameraStorage]: ...
+        schema: Optional[JsonSchemaForm] = None,
+    ) -> SC: ...
+    def get_camera_storage(self, camera_id: str) -> Optional[SC]: ...
     def remove_camera_storage(self, camera_id: str) -> None: ...
 
 
@@ -1123,13 +1087,15 @@ class PluginConfigService(Protocol):
 # Plugin API
 APIEventType = Literal["finishLaunching", "shutdown"]
 
+PA = TypeVar("PA", bound=CameraStorage[Any], default=CameraStorage[Any])
+
 
 @runtime_checkable
-class PluginAPI(Protocol):
+class PluginAPI(Protocol[PA]):
     storage_path: str
     config_file: str
     config_service: PluginConfigService
-    storage_controller: StorageController
+    storage_controller: StorageController[PA]
     device_manager: DeviceManager
 
     def on(self, event: APIEventType, listener: Callback) -> "PluginAPI": ...
